# -*- coding: utf-8 -*-
"""
Очередь на базе RabbitMQ.
"""
from __future__ import print_function
from __future__ import absolute_import
from pushme.mq import interface
import pika
from pika.adapters.tornado_connection import TornadoConnection


def create_url_parameters(host, port, login, password, vhost):
    url = 'amqp://{login}:{password}@{host}:{port}/{vhost}'.format(
        login=login or 'guest',
        password=password or 'guest',
        host=host or 'localhost',
        port=port or 5672,
        vhost=vhost,
    )

    return pika.URLParameters(url)


class Queue(interface.Queue):
    """
    Создание выделенной очереди в RabbitMQ
    """
    name = 'pushme'

    def __init__(self, *args, **kwargs):
        super(Queue, self).__init__(*args, **kwargs)
        self._connection = TornadoConnection(
            parameters=self._create_parameters(),
            on_open_callback=self.on_connection_open,
        )

    def _create_parameters(self):
        return create_url_parameters(
            *self._addr,
            *self._access,
        )

    def on_connection_open(self, connection):
        connection.channel(on_open_callback=self.on_open_channel)

    def on_open_channel(self, channel):
        channel.queue_declare(
            queue=Queue.name, durable=True, exclusive=False,
            auto_delete=False, callback=self.on_open_queue)

    def on_open_queue(self, channel):
        pass

    def run(self):
        if not self._quiet:
            print("RabbitMQ queue '%s' started at %s" % (
                self.name,
                self._connection.params,
            ))
        self._connection.ioloop.start()


class Client(interface._Client):
    """
    Базовый класс клиента очереди
    """
    def _create_parameters(self):
        return create_url_parameters(
            *self._queue_addr,
            *self._queue_access,
        )


class Sender(interface.Sender, Client):
    """
    Отправитель
    """
    exchange = 'pushme-exchange'

    def __init__(self, *args, **kwargs):
        super(Sender, self).__init__(*args, **kwargs)
        self._connection = pika.BlockingConnection(self._create_parameters())
        self._channel = self._connection.channel()
        self._channel.exchange_declare(
            exchange=self.exchange, exchange_type='fanout')
        self._channel.queue_bind(
            exchange=self.exchange, queue=Queue.name)

    def send(self, data, uid=None, topic=None):
        message = (
            '%s|%s|%s' % (
                uid or 0,
                topic or "",
                data
        )).encode()
        self._channel.basic_publish(
            self.exchange, '', message,
            pika.BasicProperties(
                content_type='text/plain', delivery_mode=1)
        )


class Receiver(interface.Receiver, Client):
    """
    Получатель
    """
    def __init__(self, *args, **kwargs):
        super(Receiver, self).__init__(*args, **kwargs)
        self._channel = None
        self._connection = TornadoConnection(
            self._create_parameters(), self.on_connected)

    def on_connected(self, connection):
        """Вызывается, когда выполнено подключение к RabbitMQ"""
        self._connection.channel(on_open_callback=self.on_channel_open)

    def on_channel_open(self, new_channel):
        """Вызывается, когда открыт канал"""
        self._channel = new_channel
        self._channel.basic_consume(queue=Queue.name, auto_ack=True,
                                    on_message_callback=self.handle_delivery)

    def handle_delivery(self, channel, method, header, body):
        """Вызывается, когда пришло сообщение из очереди"""
        try:
            message = body.decode('utf-8')
            uid, topic, data = message.split('|', 2)
            topic = topic.strip() or None
            uid = int(uid)
        except (ValueError, TypeError):
            print("Wrong message format:", body)
        else:
            self.callback(uid, topic, data)

    def process(self):
        # Сделаем заглушку, так как работаем через TornadoAdapter, а у него
        # свой loop
        pass


